use actix_web::{error::ErrorInternalServerError, HttpResponse, Result};
use mime_guess::MimeGuess;

use crate::app_state::AppState;
use crate::s3_utils::check_file_exists;

/// Функция для обслуживания файла по заданному пути.
pub async fn serve_file(file_path: &str, state: &AppState) -> Result<HttpResponse, actix_web::Error> {
    let filepath = match file_path.ends_with("/webp") {
        true => &file_path.replace("/webp", ""),
        false => file_path,
    };
    let mut parts = filepath.split('/').collect::<Vec<&str>>(); // Explicit type annotation
    let mut file_fullpath = String::new();
    if let Some(filename) = parts.pop() {
        file_fullpath = state.get_path(filename).await.unwrap().unwrap();
    }

    // Проверяем наличие файла в Storj S3
    if !check_file_exists(&state.storj_client, &state.storj_bucket, &file_fullpath).await? {
        return Err(ErrorInternalServerError(format!("File {} not found in Storj", file_fullpath)));
    }

    // Получаем объект из Storj S3
    let get_object_output = state
        .storj_client
        .get_object()
        .bucket(&state.storj_bucket)
        .key(file_fullpath.clone())
        .send()
        .await
        .map_err(|_| ErrorInternalServerError(format!("Failed to get {} object from Storj", file_fullpath)))?;

    let data: aws_sdk_s3::primitives::AggregatedBytes = get_object_output
        .body
        .collect()
        .await
        .map_err(|_| ErrorInternalServerError("Failed to read object body"))?;

    let data_bytes = data.into_bytes();
    let mime_type = MimeGuess::from_path(&file_fullpath).first_or_octet_stream();

    Ok(HttpResponse::Ok()
        .content_type(mime_type.as_ref())
        .body(data_bytes))
}
